#include "itext.h"
#include "itext_file.h"
#include <ctype.h>

int itextCheckEuc(unsigned char *p)
{
	if( *p & 0x80 ) return 1;

	return 0;
}
char * itextStrStr(char *str0,char *str1)
{
        char *p,str[256];
        int i = 0;

        for( p=str0; *p; p++){

                if(isspace(*p)) continue;

                str[i++] = *p;
        }
        str[i] = '\0';

        if((p = strstr(str,str1)) != NULL) return p;

        return NULL;
}
static int _isEx(char *p0,char *p1)
{
		int l0,l1,i;
		char pp0,pp1;

		l0 = strlen(p0);
		l1 = strlen(p1);

		if( l0 != l1 ) return 0;

		for(i = 0; i < l0; i++){
				pp0 = *(p0 + i);
				pp1 = *(p1 + i);

				if( pp0 >= 0x41 && pp0 <= 0x5a) pp0 = pp0 + 0x20;
				if( pp1 >= 0x41 && pp1 <= 0x5a) pp1 = pp1 + 0x20;

				if( pp0 != pp1 ) return 0;
		}

		return 1;
}
ftype_t itextCheckFileType_ex(char *fname)
{
	char *p;
	int len;

	for( p = fname+(strlen(fname)-1),len = 0; *p; p--,len++){
			if( *p == '.' ) break;
			if( *p < 0x41 || *p > 0x7a ) return F_BIP;
	}
	p++;


	if(_isEx(p,"text") || _isEx(p,"txt")){
			return F_TXT;

	}else if( _isEx(p,"html") || _isEx(p,"htm") ){
			return F_HTM;

	}else if( _isEx(p,"csv") ){
			return F_CSV;

	}else if( _isEx(p,"zip") ){
			return F_ZIP;
	}else{
			return F_NIP;
	}

}
fcode_t itextCheckKanjiCode(FILE *fp,char *fbuf)
{
	fpos_t pos;
	unsigned char buf[1024];
	unsigned char *p,*pp;
	int size,flag;

	if(fp != NULL && fbuf == NULL){

		fgetpos(fp,&pos);
		size = fread((unsigned char *)buf,sizeof(unsigned char),1024,fp);
		buf[size] = '\0';

		fsetpos(fp,&pos);
		pp = buf;

	}else if(fp == NULL && fbuf != NULL){
		pp = (unsigned char *)fbuf;

	}else if(fp != NULL && fbuf != NULL){
		pp = (unsigned char *)fbuf;

	}else if(fp == NULL && fbuf == NULL){
		return _U;

	}

	flag = 0;
	for( p=pp; *p; p++ ){

		if(*p == 0x1b && flag == 0)
		       	flag++;
		else if(*p == 0x24 && flag == 1)
		       	flag++;
		else if(*p == 0x26 && flag == 1)
		       	flag++;
		else if(*p == 0x40 && flag == 2)
			return _J;

		else if(*p == 0x42 && flag == 2)
			return _J;

		else if(*p == 0x28 && flag == 2)
			flag++;

		else if(*p == 0x44 && flag == 3)
			return _J;

		else
			flag = 0;
	}

	for( p=pp; *p; p++ )
		if(*p > 0x80 && *p < 0xa0) return _S;

	for( p=pp; *p; p++ )
		if(*p == 0xfd || *p == 0xfe) return _E;

	flag = 0;
	for( p=pp; *p; p++ ){

		if(*p & 0x80)
			flag = flag ? 0 : 1;

		if(flag)
			if(*p > 0xa1 && *p < 0xdf) return _E;

	}

	flag = 0;
	for( p=pp; *p; p++ )
		if(*p > 0x20 && *p < 0x7f) flag++;

	if((flag * 100 / strlen((char *)pp)) >= 80) return _U;

	return _S;
}
ftype_t itextCheckFileType(char *fname)
{
	FILE *fp;
	char buf[2048];

	fcode_t code;

	if((fp = fopen(fname,"r")) == NULL) return F_BIP;

	memset(buf,0,2048);
	fgets(buf,sizeof(int),fp);
	if( strstr(buf,"PK") != NULL ){

			fclose(fp);
			return F_ZIP;

	}

	memset(buf,0,2048);
	fseek(fp,0L,SEEK_SET);
	fread(buf,sizeof(int),2048,fp);

	if( (strstr(buf,"<html") != NULL) || (strstr(buf,"<HTML") != NULL) ){ 

		if( (strstr(buf,"<body") != NULL) || (strstr(buf,"<BODY") != NULL) ){	

			fclose(fp);
			return F_HTM;

		}
	}

	fseek(fp,0L,SEEK_SET);
	switch((code = itextCheckKanjiCode(fp,NULL))){
		case _J:
		case _E:
		case _S:

			fclose(fp);
			return F_TXT;

		break;

		case _U:
		break;
	}

	fclose(fp);

	return itextCheckFileType_ex(fname);

}
iconv_t itextConvOpen(FILE *fp,char *buf)
{
        iconv_t con;

        switch(itextCheckKanjiCode(fp,buf)){
                case _J:
					con = iconv_open("EUC-JP","ISO2022JP");
				break;
                case _S:
					con = iconv_open("EUC-JP","SHIFT-JIS");
				break;
				case _E:
					con = NULL;
				break;
				default:
					con = NULL;
				break;
        }

        return con;
}
void itextConvClose(iconv_t con)
{
	if(con != NULL)
        	iconv_close(con);

	return;
}
int itextConvKanjiCode(iconv_t con,char *to,char *from,int size)
{

	char *inbuf = NULL; 
	char *outbuf = NULL;

	int  len,from_len;

	size_t in_len,out_len;

	if(from == NULL) return -1;

	if(con == NULL){ 

		memset(to,'\0',size + 1);
		memcpy(to,from,size); 

		return size;

	}

	from_len = size; 
	len = from_len + EUC_BUFFER;
	out_len = len; 
	in_len = from_len;
	inbuf = from; 
	outbuf = to;

	if(iconv(con,&inbuf,&in_len,&outbuf,&out_len) < 0){
		fprintf(stderr,"Error: itextConvKanjiCode()(iconv())\n");
		return -1;
	}
	len -= out_len;
	*(to+len) = '\0';

        return len;
}
int itextGetTitlePerson(char *buf,char *title,char *person)
{
	int i,j;
	char *p,tmp[256],temp[12][256];
	char *unknown = "-----";

	strcpy(title,unknown);
	strcpy(person,unknown);

	for(i = 0; i < 12 ; i++)
		memset(temp[i],'\0',256);

	i = 0; j = 0;
	for( p=buf; *p; p++ ){
		if(*p == '\r') continue;

		if(*p == '\n'){
			tmp[i++] = '\0';

			if(strncmp(tmp,"----------",10) == 0) break;

			if( tmp[0] != '\r' && tmp[0] != '\n' )
				strncpy(temp[j++],tmp,i);


			i = 0;
			continue;
		}
		tmp[i++] = *p;

		if( i > 125 ) break;
	}

	j = 0;
	for(i = 0; i < 12; i++){
		if( temp[i][0] != '\0' ){
			if( j == 0 ) 
				strcpy(title,temp[i]);
			else
				strcpy(person,temp[i]);
			j++;
		}
	}

	return 0;
}
int itextBufferAlloc(char **b,char **e,int size)
{
	if(*b != NULL){
           if((*b = (char *)realloc(*b,size)) == NULL){
                        fprintf(stderr,"False _buf_alloc(1)(realloc())\n");
                        return -1;
			}
    }else{

		if((*b = (char *)malloc(size)) == NULL){
                        fprintf(stderr,"False _buf_alloc(1)(malloc())\n");
			return -1;
           	}
                memset(*b,0,size);
        }
        if(*e != NULL){
                if((*e = (char *)realloc(*e,size + EUC_BUFFER)) == NULL){
                        fprintf(stderr,"False _buf_alloc(2)(realloc())\n");
                        return -1;
                }
        }else{
                if((*e = (char *)malloc(size + EUC_BUFFER)) == NULL){
                        fprintf(stderr,"False _buf_alloc(2)(malloc())\n");
                        return -1;
                }
                memset(*e,0,size + EUC_BUFFER);
        }
        return 0;
}
int itextBufferOver(char *buf,int len,int size)
{
	int end = len - 1;

	if(buf == NULL) return 0;

	if(*(buf+end) != '\r' && *(buf+end) != '\n' && len > size - 2)
		return 1;

	return 0;
}
int itextBufferCat(char **b,char **e,char **u,int size)
{
	int len,len0,len1;

        if(*b == NULL || *e == NULL || *u == NULL || size <= 0){
                fprintf(stderr,"False buf_cat()\n");
                exit(1);
        }

		len = (len0 = strlen(*b)) + (len1 = strlen(*e));

        if(size <= len){
                size = len + 2;
                if((*b = (char *)realloc(*b,size)) == NULL){
                        fprintf(stderr,"False buf_cat()(reallo())\n");
                        exit(1);
                }
        }

		strncat(*b,*e,len1);

        if(itextBufferAlloc(e,u,size) < 0)
                exit(1);

        strncpy(*e,*b,len);

        return len;
}
BLINE_T * itextBufferLine(char *buf,int *max)
{
	BLINE_T *head,*next,*bl;
	char *p,*pp;
	int i;

	*max = 0;

	if( (head = (BLINE_T *)malloc(sizeof(BLINE_T))) == NULL) return NULL;

	head->p = NULL;
	head->l = 0;
	head->n = NULL;

	next = head;

	pp = buf;
	i = 0;
	for( p = buf; *p; p++ ){
		i++;

		if( *p == '\n' ){

			if( (bl = (BLINE_T *)malloc(sizeof(BLINE_T))) == NULL){
				fprintf(stderr,"ERROR : bline_malloc( %d ) itext_f3.c\n",i);
				pp = p + 1;
				i = 0;
				continue;
			}

			bl->p = pp;
			bl->l = i;
			bl->n = NULL;

			next->n = bl;
			next = bl;

			if( *max < i ) *max = i;

			pp = p + 1;
			i = 0;
		}
	}

	return head;
}
BLINE_T * itextBufferGets(BLINE_T *head)
{
	static BLINE_T *bl_h = NULL;

	BLINE_T *bl;

	if( head == NULL ) return NULL;

	if( bl_h == NULL ) bl_h = head;


	if( (bl = bl_h->n) == NULL ){
		bl_h = NULL;
	   	return NULL;
	}

	bl_h = bl;

	return bl;
}
void itextBufferFree(BLINE_T *head)
{
	BLINE_T *bl,*bbl;

	if( head != NULL ){

		for( bl = head->n; bl != NULL; bl = bbl ){
			bbl = bl->n;

			if( bl ) free(bl);
		}
	}
}


			

		

